if exist ..\..\cogl\cogl-enum-types.h goto DONE_COGL_ENUMS_H
perl %1\bin\glib-mkenums --template ../../cogl/cogl-enum-types.h.in ../../cogldeprecated/cogl-clip-state.h ../../cogldeprecated/cogl-fixed.h ../../cogldeprecated/cogl-material-compat.h ../../cogldeprecated/cogl-vertex-buffer.h ../../cogldeprecated/cogl-shader.h ../../cogldeprecated/cogl-clutter.h ../../cogldeprecated/cogl-type-casts.h ../../cogldeprecated/cogl-framebuffer-deprecated.h ../../cogldeprecated/cogl-texture-deprecated.h ../../cogldeprecated/cogl-auto-texture.h ../../coglcogl1-context.h ../../coglcogl-bitmap.h ../../coglcogl-color.h ../../coglcogl-matrix.h ../../coglcogl-offscreen.h ../../coglcogl-primitives.h ../../coglcogl-texture.h ../../coglcogl-types.h ../../coglcogl.h ../../cogl/cogl-win32-renderer.h > ..\..\cogl\cogl-enum-types.h
:DONE_COGL_ENUMS_H
if exist ..\..\cogl\cogl-enum-types.c goto DONE_COGL_ENUMS_C
perl %1\bin\glib-mkenums --template ../../cogl/cogl-enum-types.c.in ../../cogldeprecated/cogl-clip-state.h ../../cogldeprecated/cogl-fixed.h ../../cogldeprecated/cogl-material-compat.h ../../cogldeprecated/cogl-vertex-buffer.h ../../cogldeprecated/cogl-shader.h ../../cogldeprecated/cogl-clutter.h ../../cogldeprecated/cogl-type-casts.h ../../cogldeprecated/cogl-framebuffer-deprecated.h ../../cogldeprecated/cogl-texture-deprecated.h ../../cogldeprecated/cogl-auto-texture.h ../../coglcogl1-context.h ../../coglcogl-bitmap.h ../../coglcogl-color.h ../../coglcogl-matrix.h ../../coglcogl-offscreen.h ../../coglcogl-primitives.h ../../coglcogl-texture.h ../../coglcogl-types.h ../../coglcogl.h ../../cogl/cogl-win32-renderer.h > ..\..\cogl\cogl-enum-types.c
:DONE_COGL_ENUMS_C
